@@ -107,6 +107,7 @@ impl MpegBox for FtypBox {
 /// Metadata box
 #[derive(Debug, Clone)]
 pub struct MetaBox {
+    pub hdlr: HdlrBox,
     pub iloc: IlocBox,
     pub iinf: IinfBox,
     pub pitm: PitmBox,
@@ -118,6 +119,7 @@ impl MpegBox for MetaBox {
     #[inline]
     fn len(&self) -> usize {
         FULL_BOX_SIZE
+            + self.hdlr.len()
             + self.pitm.len()
             + self.iloc.len()
             + self.iinf.len()
@@ -128,6 +130,7 @@ impl MpegBox for MetaBox {
     fn write<B: WriterBackend>(&self, w: &mut Writer<B>) -> Result<(), B::Error> {
         let mut b = w.new_box(self.len());
         b.full_box(*b"meta", 0)?;
+        self.hdlr.write(&mut b)?;
         self.pitm.write(&mut b)?;
         self.iloc.write(&mut b)?;
         self.iinf.write(&mut b)?;
@@ -148,7 +151,7 @@ impl MpegBox for IinfBox {
     #[inline]
     fn len(&self) -> usize {
         FULL_BOX_SIZE
-        + 2 // num entries
+        + 2 // num items u16
         + self.items.iter().map(|item| item.len()).sum::<usize>()
     }
 
@@ -192,6 +195,28 @@ impl MpegBox for InfeBox {
     }
 }
 
+#[derive(Debug, Clone)]
+pub struct HdlrBox {
+}
+
+impl MpegBox for HdlrBox {
+    #[inline(always)]
+    fn len(&self) -> usize {
+        FULL_BOX_SIZE + 4 + 4 + 12
+    }
+
+    fn write<B: WriterBackend>(&self, w: &mut Writer<B>) -> Result<(), B::Error> {
+        let mut b = w.new_box(self.len());
+        b.full_box(*b"hdlr", 0)?;
+        b.u32(0)?; // ???
+        // because an image format needs to be told it's an image format
+        b.push(b"pict")?;
+        b.u32(0)?; // ???
+        b.u32(0)?; // ???
+        b.u32(0)?; // ???
+        Ok(())
+    }
+}
 /// Item properties + associations
 #[derive(Debug, Clone)]
 pub struct IprpBox {
@@ -215,32 +240,63 @@ impl MpegBox for IprpBox {
     }
 }
 
+#[derive(Debug, Clone)]
+pub enum IpcoProp {
+    Av1C(Av1CBox),
+    Pixi(PixiBox),
+    Ispe(IspeBox),
+    AuxC(AuxCBox),
+}
+
+impl IpcoProp {
+    pub fn len(&self) -> usize {
+        match self {
+            Self::Av1C(p) => p.len(),
+            Self::Pixi(p) => p.len(),
+            Self::Ispe(p) => p.len(),
+            Self::AuxC(p) => p.len(),
+        }
+    }
+
+    pub fn write<B: WriterBackend>(&self, w: &mut Writer<B>) -> Result<(), B::Error> {
+        match self {
+            Self::Av1C(p) => p.write(w),
+            Self::Pixi(p) => p.write(w),
+            Self::Ispe(p) => p.write(w),
+            Self::AuxC(p) => p.write(w),
+        }
+    }
+}
+
 /// Item Property Container box
 #[derive(Debug, Clone)]
 pub struct IpcoBox {
-    pub av1c: ArrayVec<Av1CBox, 2>,
-    pub ispe: IspeBox,
-    pub auxc: Option<AuxCBox>,
+    props: ArrayVec<IpcoProp, 6>,
+}
+
+impl IpcoBox {
+    pub fn new() -> Self {
+        Self { props: ArrayVec::new() }
+    }
+
+    pub fn push(&mut self, prop: IpcoProp) -> u8 {
+        self.props.push(prop);
+        self.props.len() as u8 // the spec wants them off by one
+    }
 }
 
 impl MpegBox for IpcoBox {
     #[inline]
     fn len(&self) -> usize {
         BASIC_BOX_SIZE
-            + self.ispe.len()
-            + self.av1c.iter().map(|a| a.len()).sum::<usize>()
-            + self.auxc.map_or(0, |a| a.len())
+            + self.props.iter().map(|a| a.len()).sum::<usize>()
     }
 
     fn write<B: WriterBackend>(&self, w: &mut Writer<B>) -> Result<(), B::Error> {
         let mut b = w.new_box(self.len());
         b.basic_box(*b"ipco")?;
-        self.ispe.write(&mut b)?;
-        for a in self.av1c.iter() {
-            a.write(&mut b)?;
-        }
-        if let Some(a) = &self.auxc {
-            a.write(&mut b)?;
+        for p in self.props.iter() {
+            p.write(&mut b)?;
         }
         Ok(())
     }
@@ -264,29 +320,29 @@ impl AuxCBox {
     }
 }
 
-// /// Pixies, I guess.
-// #[derive(Debug, Copy, Clone)]
-// pub struct PixiBox {
-//     depth: u8,
-//     channels: u8,
-// }
+/// Pixies, I guess.
+#[derive(Debug, Copy, Clone)]
+pub struct PixiBox {
+    pub depth: u8,
+    pub channels: u8,
+}
 
-// impl PixiBox {
-//     pub fn len(&self) -> usize {
-//         BASIC_BOX_SIZE
-//             + 1 + self.channels as usize
-//     }
+impl PixiBox {
+    pub fn len(&self) -> usize {
+        FULL_BOX_SIZE
+            + 1 + self.channels as usize
+    }
 
-//     pub fn write<B: WriterBackend>(&self, w: &mut Writer<B>) -> Result<(), B::Error> {
-//         let mut b = w.new_box(self.len());
-//         b.basic_box(*b"pixi")?;
-//         b.u8(self.channels)?;
-//         for _ in 0..self.channels {
-//             b.u8(self.depth)?;
-//         }
-//         Ok(())
-//     }
-// }
+    pub fn write<B: WriterBackend>(&self, w: &mut Writer<B>) -> Result<(), B::Error> {
+        let mut b = w.new_box(self.len());
+        b.full_box(*b"pixi", 0)?;
+        b.u8(self.channels)?;
+        for _ in 0..self.channels {
+            b.u8(self.depth)?;
+        }
+        Ok(())
+    }
+}
 
 /// This is HEVC-specific and not for AVIF, but Chrome wants it :(
 #[derive(Debug, Copy, Clone)]
